import typing as typ

HOST: str = "34.232.126.251"
PORT: str = "5432"
USER: str = "pacificoadmin"
PASSWORD: str = "PacificoR2020"

SQL_MODIFICATION_WORDS: typ.Set[str] = {"UPDATE", "DELETE", "INSERT", "CREATE", "ALTER", "CREATE", "DROP"}
