def getToken() -> str:
    """
    Returns token for API.

    Returns (str): API token.

    """
    return "KmK9FqgJ5h1pSNjh1vhha6ul30vXBOBiMVlmplXf"

if __name__ == '__main__':
    import datetime as dt

    dt.timedelta()